<?php
/*
  Plugin Name: WooCommerce OpenPay CreditCard Payment Gateway
  Plugin URI: https://www.twv.com.tw/
  Description: OpenPay CreditCard Payment Gateway allows you to use TWV OpenPay CreditCard payment service.
  Version: 2.1.14d
  Author: TWV OpenPay DevTeam
  Author URI: https://www.twv.com.tw/
*/

if ( ! defined( 'ABSPATH' ) )
  exit;

add_action('plugins_loaded', 'wc_openpay_ccard_init', 0);

function wc_openpay_ccard_init() {

  if ( !class_exists( 'WC_Payment_Gateway' ) ) return;

  /**
    * Gateway class
    */
  class WC_Payment_Gateway_Openpay_Ccard extends WC_Payment_Gateway {
    protected $pay_url          = 'https://www.twv.com.tw/openpay/pay.php';
    protected $select_paymethod = "1";
    protected $iid              = "0";

    public function __construct(){
      $this->id                 = 'openpay_ccard_gateway';
      $this->has_fields         = false;
      $this->method_title       = '台灣里信用卡付款';
      $this->class_name         = get_class($this);

      $this->init_form_fields();
      $this->init_settings();

      // Define user set variables
      $this->title        = $this->get_option( 'title' );
      $this->description  = $this->get_option( 'description' );
      $this->openpay_mid  = $this->get_option( 'openpay_mid' );
      $this->openpay_key1 = $this->get_option( 'openpay_key1' );
      $this->openpay_key2 = $this->get_option( 'openpay_key2' );

      // Actions
      add_action( 'woocommerce_update_options_payment_gateways_'.$this->id, array( $this, 'process_admin_options' ) );
      add_action( 'woocommerce_receipt_'.$this->id,  array($this, 'do_receipt'));
      add_action( 'woocommerce_thankyou_'.$this->id, array($this, 'do_thankyou'));

      add_action( 'woocommerce_api_'.strtolower($this->class_name), array(&$this, 'handle_callback') );
    }

    /**
     * Admin Panel Options
     **/
    public function admin_options(){
      print '<h3>台灣里信用卡付款</h3>';
      print '<p>台灣里信用卡付款模組提供您使用台灣里的付款服務.</p>';
      print '<table class="form-table">';
      $this->generate_settings_html();
      print '<tr><td>付款通知URL</td><td>'.WC()->api_request_url($this->class_name)
        .'<p class="description">台灣里於付款入帳時, 通知商家 WooCommerce 付款入帳修改訂單狀態的通知接收網址. 啟用多個台灣里付款模組時, 只需設定任意一組通知網址即可即時接收入帳通知.</p>'
        .'</td></tr>'."\n";
      print '</table>';
    }

    /**
     * Initialise Gateway Settings Form Fields
    **/
    function init_form_fields(){
      $this->form_fields = array(
        'enabled' => array(
          'title'       => 'Enable/Disable',
          'type'        => 'checkbox',
          'label'       => '啟用台灣里信用卡付款',
          'description' => '啟用或禁用此付款方式.',
          'desc_tip'    => true,
          'default'     => 'yes'
        ),
        'title' => array(
          'title'       => '結帳標題',
          'type'        => 'text',
          'description' => '結帳時本付款方式標題.',
          'desc_tip'    => false,
          'default'     => '台灣里信用卡付款'
        ),
        'description' => array(
          'title'       => '結帳說明',
          'type'        => 'textarea',
          'description' => '結帳時本付款方式說明.',
          'default'     => '使用台灣里信用卡付款'
        ),
        'openpay_mid' => array(
          'title'       => '台灣里 Merchant ID',
          'type'        => 'text',
          'description' => '台灣里商家編號',
          'default'     => 'TEST',
          'desc_tip'    => false
        ),
        'openpay_key1' => array(
          'title'       => '台灣里 Key1',
          'type'        => 'text',
          'description' => '台灣里商家金鑰1(key1)',
          'default'     => '21216879a5d9e1c51b3cbc1c68b22399',
          'desc_tip'    => false
        ),
        'openpay_key2' => array(
          'title'       => '台灣里 Key2',
          'type'        => 'text',
          'description' => '台灣里商家金鑰2(key2)',
          'default'     => '6d4b111610073f9c1105d3f852a3d039',
          'desc_tip'    => false
        )
      );
    }

    function _openpay_pay_request_handling( $order ) {

      $req_url = $this->pay_url;
      $req_param = array();
      $req_param['version']          = '2.1';
      $req_param['mid']              = $this->openpay_mid;
      $req_param['txid']             = $order->id;
      $req_param['amount']           = ceil( $order->order_total );
      $req_param['verify']           = md5(implode( '|', array( $this->openpay_key1, $this->openpay_mid, $req_param['txid'], $req_param['amount'], $this->openpay_key2) ));
      $req_param['return_url']       = $this->get_return_url( $order );
      $req_param['select_paymethod'] = $this->select_paymethod;
      $req_param['iid']              = $this->iid;
      $req_param['language']         = get_locale() == "zh_TW" ? "tchinese" : "english";
      $req_param['charset']          = "UTF-8";
      $req_param['cart_type']        = "woocommerce".WC()->version."-openpay2.1.14d";
      $req_param['cname']            = $order->billing_last_name . $order->billing_first_name;
      $req_param['caddress']         = $order->billing_postcode . $order->billing_state . $order->billing_city . $order->billing_address_1 . $order->billing_address_2;
      $req_param['ctel']             = $order->billing_phone;
      $req_param['cemail']           = $order->billing_email;

      return array( $req_url, $req_param );
    }

    function _openpay_pay_result_handling( $order_id, $res_param ) {
      try {
        $order = wc_get_order( $order_id );

        if( ! $order ) {
          return "order not found (".$order_id.")";
        }

        if( empty($res_param['verify']) ) {
          throw new Exception( "error empty param [verify]" );
        }

        $digest_text  = implode( '|', array( $this->openpay_key1, $res_param['txid'], $res_param['amount'], $res_param['pay_type'], $res_param['status'], $res_param['tid'], $this->openpay_key2) );
        $digest_value = md5( $digest_text );

        if( $digest_value !== $res_param['verify'] ) {
          throw new Exception( "error empty param [verify]" );
        }

        if( $order_id != $res_param['txid'] ) {
          throw new Exception( "error order_id mismatch" );
        }

        if( ceil( $order->order_total ) != $res_param['amount'] ) {
          throw new Exception( "error amount mismatch" );
        }

        switch( $res_param['pay_type'] ) {
        case '1' : $pay_type = '信用卡'; break;
        case '2' : $pay_type = '銀行ATM轉帳'; break;
        case '4' : $pay_type = 'WebATM'; break;
        case '8' : $pay_type = '超商條碼'; break;
        case '9' : $pay_type = 'ibon'; break;
        case '12': $pay_type = 'FamiPort'; break;
        case '16': $pay_type = '7-11 取貨付款'; break;
        case '17': $pay_type = '銀聯(UPOP)'; break;
        default:   $pay_type = '其它('.$res_param['pay_type'].')';
        }

        switch( $res_param['status'] ) {
        case 1:
          $result_status = 1;
          $result_msg = "台灣里交易單號:".$res_param['tid']."<br>付款方式:".$pay_type."<br>付款結果:付款成功";
          break;
        case 2:
          $result_status = 2;
          $result_msg = "台灣里交易單號:".$res_param['tid']."<br>付款方式:".$pay_type."<br>付款結果:付款失敗,".$res_param['error_desc'];
          break;
        case 3:
          $result_status = 3;
          $result_msg = "台灣里交易單號:".$res_param['tid']."<br>付款方式:".$pay_type."<br>付款結果:等待付款"
                       ."<br><a href='".$res_param['bill_url']."' target='_blank'>顯示繳費單</a>";
          break;
        default:
          $result_status = 2;
          $result_msg = "台灣里交易單號:".$res_param['tid']."<br>付款方式:".$pay_type."<br>付款結果:付款失敗,回傳狀態錯誤(".$res_param['status'].")";
        }

      } catch ( Exception $e ) {
        $result_status = 2;
        $result_msg = "付款失敗,檢查錯誤:(".$e->getMessage().")";
      }

      switch( $result_status ) {
      case 1:
        switch( $order->get_status() ) {
        case 'pending':
        case 'on-hold':
        case 'failed':
          // Update order status
          $order->update_status( 'processing', '付款成功, 訂單處理中.' );
          break;
        }

        //Add customer order note
        $order->add_order_note( $result_msg, 1 );

        // Reduce stock levels
        $order->reduce_order_stock();
        // Empty cart
        WC()->cart->empty_cart();

        break;
      case 3:
        switch( $order->get_status() ) {
        case 'pending':
          // Update order status
          $order->update_status( 'on-hold', '等待付款中.' );
          break;
        }

        //Add customer order note
        $order->add_order_note( $result_msg, 1 );

        // Reduce stock levels
        $order->reduce_order_stock();
        // Empty cart
        WC()->cart->empty_cart();

        break;
      default:
        $order->update_status('failed', '付款失敗');

        //Add customer order note
        $order->add_order_note( $result_msg, 1 );

      }

      return $result_msg;
    }

    /**
     * Process the payment and return the result
    **/
    function process_payment( $order_id ) {
      $order        = wc_get_order( $order_id );

      return array(
        'result'   => 'success',
        'redirect' => $order->get_checkout_payment_url( true )
      );
    }

    /**
     * Output for the order received page.
    **/
    function do_receipt( $order_id ) {
      $order   = wc_get_order( $order_id );

      list( $req_url, $req_param ) = $this->_openpay_pay_request_handling( $order );

      $form_html  = '<form id="openpay_payment_form" method="post" action="'.esc_url( $req_url ).'" target="_top">'."\n";
      foreach( $req_param as $k => $v ) {
        $form_html .= '<input type="hidden" name="'.esc_attr( $k ).'" value="'.esc_attr( $v ).'">'."\n";
      }
      $form_html .= '<!-- Button Fallback -->
          <div class="payment_buttons">
            <input type="submit" class="button alt" id="submit_openpay_payment_form" value="下一步,前往付款" />
          </div>
        </form>';

      print '<p>已記錄訂單明細. 接下來請點選 [下一步,前往付款] 按鈕繼續付款流程.</p>';
      print $form_html;
    }

    /**
     * verify payment result
    **/
    function do_thankyou( $order_id ) {
      $result_msg = $this->_openpay_pay_result_handling( $order_id, $_POST );

      print "<fieldset><legend>台灣里付款結果</legend>".$result_msg."</fieldset>";
      return;

    }

    function handle_callback() {
      print "OK<br>\n";
      $order_id = $_REQUEST['txid'];
      $result_msg = $this->_openpay_pay_result_handling( $order_id, $_REQUEST );
      print $result_msg;
      exit;
    }

  }

  /**
   * Add Payment Gateway to WC
   **/
  function wc_add_gateway_openpay_ccard($methods) {
    $methods[] = 'WC_Payment_Gateway_Openpay_Ccard';
    return $methods;
  }

  add_filter('woocommerce_payment_gateways', 'wc_add_gateway_openpay_ccard' );

}
