<?php
/*
  Plugin Name: WooCommerce OpenPay 7-11 Cod Shipment
  Plugin URI: https://www.twv.com.tw/
  Description: OpenPay 7-11 Cod Shipment allows you to use TWV OpenPay 7-11 Cod Shipment & Payment.
  Version: 2.1.10
  Author: TWV OpenPay DevTeam
  Author URI: https://www.twv.com.tw/
*/

if ( ! defined( 'ABSPATH' ) )
  exit;

add_action('plugins_loaded', 'wc_shipping_method_openpay_pcsc_init', 0);

function wc_shipping_method_openpay_pcsc_init() {

  if ( !class_exists( 'WC_Shipping_Method' ) ) return;

  /**
    * Gateway class
    */
  class WC_Shipping_Method_Openpay_Pcsc extends WC_Shipping_Method {
    public function __construct(){
      $this->id                 = 'opepay_pcsc_shippig';
      $this->method_title       = 'OpenPay 7-11 取貨';

      $this->init_form_fields();
      $this->init_settings();

      // Define user set variables
      $this->title        = $this->get_option( 'title' );
      $this->description  = $this->get_option( 'description' );
      $this->service_fee  = $this->get_option( 'service_fee' );

      // Actions
      add_action('woocommerce_update_options_shipping_' . $this->id, array( $this, 'process_admin_options' ), 1 );
    }

    /**
     * Admin Panel Options
     **/
    public function admin_options(){
      print '<h3>OpenPay WebATM Woocommerce Payment Gateway</h3>';
      print '<p>提供您使用台灣里 7-11 取貨付款附加配送費用.</p>';
      print '<table class="form-table">';
      $this->generate_settings_html();
      print '</table>';
    }

    /**
     * Initialise Gateway Settings Form Fields
    **/
    function init_form_fields(){
      $this->form_fields = array(
        'enabled' => array(
          'title'       => 'Enable/Disable',
          'type'        => 'checkbox',
          'label'       => '啟用 OpenPay 7-11 取貨',
          'description' => '啟用或禁用此運送方式.',
          'desc_tip'    => true,
          'default'     => 'yes'
        ),
        'title' => array(
          'title'       => '運送方式標題',
          'type'        => 'text',
          'description' => '結帳時本運送方式標題.',
          'desc_tip'    => false,
          'default'     => 'OpenPay 7-11 取貨'
        ),
        'service_fee' => array(
          'title'       => '運送附加費用',
          'type'        => 'text',
          'description' => '結帳時本運送方式運送附加費用.',
          'desc_tip'    => false,
          'default'     => '60'
        )
      );
    }

    public function calculate_shipping() {
      $rate = array(
        'id'    => $this->id,
        'label' => $this->title,
        'cost'  => $this->service_fee
      );
      $this->add_rate( $rate );
    }
  }

  /**
   * Add Payment Gateway to WC
   **/
  function wc_add_shipping_method_openpay_pcsc($methods) {
    $methods[] = 'WC_Shipping_Method_Openpay_Pcsc';
    return $methods;
  }

  add_filter( 'woocommerce_shipping_methods', 'wc_add_shipping_method_openpay_pcsc' );
}
